'use strict';

const ofetch = require('ofetch');
const h3 = require('h3');
const environments = require('vitest/environments');
const index$1 = require('unenv/runtime/fetch/index');
require('path');
require('fs');
const module$1 = require('module');

const TRAILING_SLASH_RE = /\/$|\/\?/;
function hasTrailingSlash(input = "", queryParameters = false) {
  if (!queryParameters) {
    return input.endsWith("/");
  }
  return TRAILING_SLASH_RE.test(input);
}
function withTrailingSlash(input = "", queryParameters = false) {
  if (!queryParameters) {
    return input.endsWith("/") ? input : input + "/";
  }
  if (hasTrailingSlash(input, true)) {
    return input || "/";
  }
  const [s0, ...s] = input.split("?");
  return s0 + "/" + (s.length > 0 ? `?${s.join("?")}` : "");
}
function hasLeadingSlash(input = "") {
  return input.startsWith("/");
}
function withoutLeadingSlash(input = "") {
  return (hasLeadingSlash(input) ? input.slice(1) : input) || "/";
}
function isNonEmptyURL(url) {
  return url && url !== "/";
}
function joinURL(base, ...input) {
  let url = base || "";
  for (const index of input.filter((url2) => isNonEmptyURL(url2))) {
    url = url ? withTrailingSlash(url) + withoutLeadingSlash(index) : index;
  }
  return url;
}

module$1.createRequire((typeof document === 'undefined' ? require('u' + 'rl').pathToFileURL(__filename).href : (document.currentScript && document.currentScript.src || new URL('index.cjs', document.baseURI).href)));

function importModule(path) {
  return import(path).then((i) => {
    if (i && i.default && i.default.__esModule)
      return i.default
    return i
  })
}

const happyDom = (async function(_, { happyDom = {} }) {
  const { Window, GlobalWindow } = await importModule("happy-dom");
  const window = new (GlobalWindow || Window)(happyDom);
  return {
    window,
    teardown() {
      window.happyDOM.cancelAsync();
    }
  };
});

const jsdom = (async function(global, { jsdom = {} }) {
  const {
    CookieJar,
    JSDOM,
    ResourceLoader,
    VirtualConsole
  } = await importModule("jsdom");
  const {
    html = "<!DOCTYPE html>",
    userAgent,
    url = "http://localhost:3000",
    contentType = "text/html",
    pretendToBeVisual = true,
    includeNodeLocations = false,
    runScripts = "dangerously",
    resources,
    console = false,
    cookieJar = false,
    ...restOptions
  } = jsdom;
  const window = new JSDOM(
    html,
    {
      pretendToBeVisual,
      resources: resources ?? (userAgent ? new ResourceLoader({ userAgent }) : void 0),
      runScripts,
      url,
      virtualConsole: console && global.console ? new VirtualConsole().sendTo(global.console) : void 0,
      cookieJar: cookieJar ? new CookieJar() : void 0,
      includeNodeLocations,
      contentType,
      userAgent,
      ...restOptions
    }
  ).window;
  window.scrollTo = () => {
  };
  return {
    window,
    teardown() {
    }
  };
});

const index = {
  name: "nuxt",
  async setup(global, environmentOptions) {
    const url = joinURL("http://localhost:3000", environmentOptions?.nuxtRuntimeConfig.app?.baseURL || "/");
    const { window: win, teardown } = await {
      "happy-dom": happyDom,
      jsdom
    }[environmentOptions.nuxt.domEnvironment || "happy-dom"](global, {
      ...environmentOptions,
      happyDom: {
        url,
        ...environmentOptions?.happyDom
      },
      jsdom: {
        url,
        ...environmentOptions?.jsdom
      }
    });
    win.__NUXT__ = {
      serverRendered: false,
      config: {
        public: {},
        app: { baseURL: "/" },
        ...environmentOptions?.nuxtRuntimeConfig
      },
      data: {},
      state: {}
    };
    const app = win.document.createElement("div");
    app.id = environmentOptions.nuxt.rootId;
    win.document.body.appendChild(app);
    win.IntersectionObserver = win.IntersectionObserver || class IntersectionObserver {
      observe() {
      }
      unobserve() {
      }
      disconnect() {
      }
    };
    const h3App = h3.createApp();
    if (!win.fetch) {
      await import('whatwg-fetch');
    }
    const localCall = index$1.createCall(h3.toNodeListener(h3App));
    const localFetch = index$1.createFetch(localCall, win.fetch);
    const registry = /* @__PURE__ */ new Set();
    win.fetch = (init, options) => {
      if (typeof init === "string" && registry.has(init)) {
        init = "/_" + init;
      }
      return localFetch(init, options);
    };
    win.$fetch = ofetch.createFetch({ fetch: win.fetch, Headers: win.Headers });
    win.__registry = registry;
    win.__app = h3App;
    const { keys, originals } = environments.populateGlobal(global, win, {
      bindFunctions: true
    });
    await import('#app/entry').then((r) => r.default());
    return {
      // called after all tests with this env have been run
      teardown() {
        teardown();
        keys.forEach((key) => delete global[key]);
        originals.forEach((v, k) => global[k] = v);
      }
    };
  }
};

module.exports = index;
